; Only one instance of the program can be running at a time.
#SingleInstance Force
; SetWinDelay, 0 is necessary to make the semi-transparent background window move without delay.
SetWinDelay, 0
; Tell the program what to do if it is closed.
OnExit, ExitSequence
; Change the name of the program in the tray menu, then remove the standard tray items
; and add the ones relevant to this program.
SplitPath, A_ScriptName,,,,ScriptNameNoExt
menu, tray, tip, %ScriptNameNoExt%
menu, tray, NoStandard
menu, tray, Add, Restart Script, RestartSequence
menu, tray, Add, Exit, ExitSequence
ScriptNameNoExtLength := StrLen(ScriptNameNoExt)
if ScriptNameNoExtLength >= 199
	gosub ErrorFilenameLength
; Enable debug functions if they exist and if the program is not compiled.
; This way normal users can't (accidentally) activate them.
If (IsLabel("DebugFunctions") AND A_IsCompiled != 1)
	gosub DebugFunctions
; Initialise the settings. To make sure all settings exist, first load the default settings.

gosub DefaultSettings
SettingsFileName = %ScriptNameNoExt% Config`.dll

ifExist,%SettingsFileName%
	gosub ReadSettingsFile
gosub SetVariablesDependentOnSettings
; Install and configure the icons.
Fileinstall, Icons.icl, Icons.icl, 1
ImageFilename = Icons.icl
menu, tray, icon, %ImageFilename%, 3, 1

goto CreateArrays

CreateArrays:
; Create the array with the icon indexes from the icons library. The icon names are used in the requirement lists and translated into the actual icons using this array.
IconArray := {BProgress:01, BSProgress:02, Bully:03, Bully2:04, BullyMissions:05, BullyErrands:06, BullyPRoute:07, BullySoda:08, BullySodaPurchase:09, BullyValeBR:10, BullyTownBR:11, BullyCoveBR:12, BullyAcaBR:13, BullyGPR:14, BullyGOStreet:15, BullyMonkeyHS:16, BullyYearbook:17, BullyChem:18, BullyPhotoClass:19, BullyShop:20, BullyPump:21, BullyBoxing:22, BullyCards:23, BullyCarGames:24, BullyCarnRides:25, BullyArt:26, BullyBiology:27, BullyEnglish:28, BullyGeo:29, BullyGym:30, BullyMath:31, BullyMusic:32, BullyClo:33, BullySumoHS:34, BullyDunce:35, BullyFireAlarm:36, BullyGnome:37, BullyKeepUp:38, BullyLockerClo:39, BullyLMHouse:40, BullyLMPark:41, BullyNutHS:42, BullyOnFoot:43, BullyPenShoot:44, BullyProjHits:45, BullyRubber:46, BullyPirate:47, BullyTomb:48, BullyRadio:49, BullyOnBike:50}
goto WelcomeScreen

; These default settings will be used if they haven't been configured otherwise in the settings file.
DefaultSettings:
TextColour = 000000
BackColour = F1F2F3
MaximumRowsText = 24
MaximumRowsIcons = 12
DecimalPlaces = 2
RefreshRate = 500
RefreshRateFileOutput = 10000
Transparency = 255
TextSmoothing = 1
OutputWindowBoldText = 0
ShowDoneIfDone = 0
ExitConfirmed = 0
AlwaysOnTop = 0
return

; Some of the settings have to be explicitly activated, or other variables depend on them.
SetVariablesDependentOnSettings:
;IconListViewWidth := 80
;TextListViewWidth := 230
If OutputWindowBoldText = 1
	CharacterWidth := 6
Else
	CharacterWidth := 5.5
;IconListViewWidthWithFloat := IconListViewWidth+5*DecimalPlaces
; For text mode the list view width is determined more precisely while it is created
; so defining a width with float here is not necessary.
SetFormat, Float, 0.%DecimalPlaces%
return

; Save the settings to the settings file.
WriteSettingsFile:
IniWrite, %TextColour%, %SettingsFileName%, Options, Text colour
IniWrite, %BackColour%, %SettingsFileName%, Options, Background colour
IniWrite, %MaximumRowsText%, %SettingsFileName%, Options, Text mode maximum rows
IniWrite, %MaximumRowsIcons%, %SettingsFileName%, Options, Icon mode maximum rows
IniWrite, %DecimalPlaces%, %SettingsFileName%, Options, Decimal places
/*
IniWrite, %RefreshRate%, %SettingsFileName%, Options, Refresh rate (ms)
*/
RefreshRateFileOutput := Round(RefreshRateFileOutput/1000)
IniWrite, %RefreshRateFileOutput%, %SettingsFileName%, Options, Refresh rate file output (sec)
RefreshRateFileOutput := Round(RefreshRateFileOutput*1000)
Transparency := 100-Round(Transparency/2.54)
IniWrite, %Transparency%, %SettingsFileName%, Options, Transparency
Transparency := 255-Round(Transparency*2.54)
IniWrite, %TextSmoothing%, %SettingsFileName%, Options, Text Smoothing
IniWrite, %OutputWindowBoldText%, %SettingsFileName%, Options, Bold Output Text
IniWrite, %ShowDoneIfDone%, %SettingsFileName%, Options, Show Done If Done
IniWrite, %ExitConfirmed%, %SettingsFileName%, Options, Disable Exit Confirmation
IniWrite, %AlwaysOnTop%, %SettingsFileName%, Options, Always On Top
return

; Read the settings from the settings file.
ReadSettingsFile:
IniRead, TextColour, %SettingsFileName%, Options, Text colour
IniRead, BackColour, %SettingsFileName%, Options, Background colour
IniRead, MaximumRowsText, %SettingsFileName%, Options, Text mode maximum rows
IniRead, MaximumRowsIcons, %SettingsFileName%, Options, Icon mode maximum rows
IniRead, DecimalPlaces, %SettingsFileName%, Options, Decimal places
/*
IniRead, RefreshRate, %SettingsFileName%, Options, Refresh rate (ms)
*/
IniRead, RefreshRateFileOutput, %SettingsFileName%, Options, Refresh rate file output (sec)
RefreshRateFileOutput := Round(RefreshRateFileOutput*1000)
IniRead, Transparency, %SettingsFileName%, Options, Transparency
Transparency := 255-Round(Transparency*2.54)
IniRead, TextSmoothing, %SettingsFileName%, Options, Text Smoothing
IniRead, OutputWindowBoldText, %SettingsFileName%, Options, Bold Output Text
IniRead, ShowDoneIfDone, %SettingsFileName%, Options, Show Done If Done
IniRead, ExitConfirmed, %SettingsFileName%, Options, Disable Exit Confirmation
IniRead, AlwaysOnTop, %SettingsFileName%, Options, Always On Top
return

ErrorFilenameLength:
Gui, FilenameError:Add, Text,, The filename of the 100`% Checklist is too long: `n`nThe program can still function but settings cannot `nbe saved and file output is unavailable. `nDo you still want to continue`?
Gui, FilenameError:Add, Text,h0 w0 Y+4,
Gui, FilenameError:Add, Button, section default, Continue
Gui, FilenameError:Add, Button, ys, Abort
Gui, FilenameError:Show
Pause, On
return



FilenameErrorButtonContinue:
FilenameErrorGuiClose:
FilenameErrorGuiEscape:
Pause, Off
Gui, FilenameError:Destroy
return

FilenameErrorButtonAbort:
ExitConfirmed = 1
ExitApp

; ######################################################################################################
; ########################################### WELCOME WINDOW ###########################################
; ######################################################################################################

/*
Subheadings:

	WelcomeScreen
	2GuiClose/2GuiEscape/2ButtonClose
	2GuiContextMenu
	2ButtonConfirm
	SelectGameCode
	SetCurrentGameCode
*/

; Create the welcome window. First the minimize and maximize buttons are removed, and LastFound is set so
; functions affecting the window later will automatically act on this one without having to specify.
; Next the background colour is set and then, dependent on the settings, the font options are set.
; If the window is even slightly transparent, the font gets maximum boldness, and if text smoothing
; is off or if the window is again even slightly transparent, text smoothing is turned off (having
; it turned on with a transparent background creates ugly borders). It proceeds to add the text and
; other controls to the window, some of which have somewhat specific placement settings, such as
; width or position relative to the previous control. Some empty text controls are also created to
; create some space between the controls above and below it. If any level of transparency is in play,
; the background window is made entirely transparent and some preparations are made for faking a
; semi-transparent background. After the gui is rendered, a second window is created which is locked
; to the first. It consists of just a background which is semi-transparent. (Within one window, it is
; possible to make either the background completely transparent, or the entire window, including controls,
; semi-transparent. This is the only decent solution I have found. It does have some issues (see readme)
; but nothing major.) The background window is set to be the owner of the real window, to avoid it being
; selectable which would push the background over the other window. Even if the background is completely
; transparent, the background window is still created, because otherwise clicks would fall through (this
; only happens if a specific colour is made transparent as is the case for the real window, it doesn't
; happen if the entire window is made completely transparent as is the case for the background window.)
WelcomeScreen:
gui 2:-MinimizeBox -MaximizeBox +LastFound
gui, 2:Color, %BackColour%, %BackColour%

if Transparency < 255
{
	gui, 2:Font, w1000
	WelcomeWindowWidth := 300
}
else
	WelcomeWindowWidth := 250
	
if (Transparency < 255 or TextSmoothing = 0)
	gui, 2:Font, Q3
gui, 2:Add, Text,c%TextColour% center w%WelcomeWindowWidth%, Welcome to the Bully 100`% Checklist!
gui, 2:Add, Text,c%TextColour% y+5 center w%WelcomeWindowWidth%, Created by`: LohaTronsRS
gui, 2:Add, Text,c%TextColour% y+10 center w%WelcomeWindowWidth%, Based on Lightnat0r's 100`% Checklist v1.58

if Transparency < 255
	gui, 2:Add, Text,c%TextColour% w110 xs section, Select output type:
else
	gui, 2:Add, Text,c%TextColour% w90 xs section, Select output type:
gui, 2:Add, Radio,c%TextColour% ys vOutputTypeText checked gHideOutputFileLocation, Text
gui, 2:Add, Radio,c%TextColour% ys vOutputTypeIcons gHideOutputFileLocation, Icons
gui, 2:Add, Radio,c%TextColour% ys vOutputTypeFile gShowOutputFileLocation, File
gui, 2:Add, Checkbox,c%TextColour% vPercentageOnlyMode xs, Show percentage only

OutputFileString = Output stored in `"%ScriptNameNoExt% Output`.txt`"
OutputFileStringLength := StrLen(OutputFileString)
OutputFileTextRows := Ceil(OutputFileStringLength*6/WelcomeWindowWidth) ; We need 6 pixels per character.
gui, 2:Add, Text,c%TextColour% w%WelcomeWindowWidth% r%OutputFileTextRows% xs vOutputFileText,

gui, 2:Add, Button, xs section, Confirm
gui, 2:Add, Button, ys, Close
gui, 2:Add, Button, ys, Settings

if Transparency < 255
{
	WinSet, TransColor, %BackColour%
	WelcomeWindowID := WinExist()
	OnMessage(0x0047, "WM_WELCOMEWINDOWPOSCHANGED")
	gui 2: -Border
}
gui, 2:Show
if Transparency < 255
{
	WinGetPos,,,WelcomeWindowNoBorderWidth,WelcomeWindowNoBorderHeight
	gui 2: +Border
	gui, 2:Show, Autosize
	WinGetPos,WelcomeWindowX,WelcomeWindowY,WelcomeWindowWidth,WelcomeWindowHeight
	BackgroundX := WelcomeWindowX+WelcomeWindowWidth-WelcomeWindowNoBorderWidth
	BackgroundY := WelcomeWindowY+WelcomeWindowHeight-WelcomeWindowNoBorderHeight
	gui 5: +LastFound
	gui, 5:Color, %BackColour%, %BackColour%
	Winset, Transparent, %Transparency%
	if AlwaysOnTop = 1
		Winset, AlwaysOnTop, On
	gui 5: -Caption
	Gui, 5: Show, x%BackgroundX% y%BackgroundY% w%WelcomeWindowNoBorderWidth% h%WelcomeWindowNoBorderHeight% NoActivate
	gui 2: +Owner5
	if AlwaysOnTop = 1
		Winset, AlwaysOnTop, On
}
return

ShowOutputFileLocation:
GuiControlGet, OutputFileTextContents, , OutputFileText
if (OutputFileTextContents != OutputFileString)
	GuiControl, , OutputFileText, %OutputFileString%
return

HideOutputFileLocation:
GuiControlGet, OutputFileTextContents, , OutputFileText
if (OutputFileTextContents != "")
	GuiControl, , OutputFileText,
return

; What happens if the user tries to close the window.
2GuiClose:
2GuiEscape:
2ButtonClose:
CurrentGUI = 2
exitapp
return

; Create a context menu if the user right clicks on the window.
2GuiContextMenu:
Menu, OutputRightClick, Add, Restart Script, RestartSequence
Menu, OutputRightClick, Add, Exit, 2GuiClose
Menu, OutputRightClick, Show
return

; This function is set to trigger as soon as the real window is moved and moves the
; background window with it.
WM_WELCOMEWINDOWPOSCHANGED()
{
	global
	gui 2: +LastFound
	WinGetPos,WelcomeWindowX,WelcomeWindowY,
	gui 5: +LastFound
	BackgroundX := WelcomeWindowX+(WelcomeWindowWidth-WelcomeWindowNoBorderWidth)
	BackgroundY := WelcomeWindowY+(WelcomeWindowHeight-WelcomeWindowNoBorderHeight)
	WinMove, BackgroundX, BackgroundY
}

; If the user presses the confirm button, first check if all the options have been
; selected correctly. If not, show an error message and restore the window. The
; program tries to restore the background window without checking if it exists, but
; this doesn't seem to cause any issues. If the options are selected correctly,
; configure the right options internally. Proceed to the output window after that.
2ButtonConfirm:
gui, 5:submit
gui, 2:submit
CurrentGame = Bully
if OutputTypeText = 1
	OutputType = Text
else if OutputTypeIcons = 1
	OutputType = Icons
else if OutputTypeFile = 1
	OutputType = File
if PercentageOnlyMode = 1
	SpecialMode = PercentageOnly
; Now that the options are selected, we can create the requirements array.
CurrentLoopCode = CreateRequirementsArray
gosub %CurrentGame%%SpecialMode%Requirements
If (OutputType = "File")
	Goto FileOutput
Else
	Goto OutputWindow

; If the settings button is pressed, hide the welcome window and start the subroutine
; which creates the settings window.
2ButtonSettings:
gui 2:Hide
gui 5:Hide
gosub CreateSettingsWindow
return

; Create the requirements array. It is initialised the first time this subroutine is
; called, then it can be populated.
CreateRequirementsArray:
if %CurrentGame%%SpecialMode%RequirementsArrayCreated != 1
{
	%CurrentGame%%SpecialMode%RequirementsArrayIndex = 1
	%CurrentGame%%SpecialMode%RequirementsArray := {}
	%CurrentGame%%SpecialMode%RequirementsArrayCreated = 1
}
else
	%CurrentGame%%SpecialMode%RequirementsArrayIndex += 1

; The following shouldn't be necessary since values in an array can be empty.
/*
if Type = 
	Type = Null
if TotalRequired = 
	TotalRequired = Null
if Address%A_Index%Length = 
	Address%A_Index%Length = Null
if Address%A_Index%CustomCode = 
	Address%A_Index%CustomCode = Null
*/

; Create the array with all the requirements. Each name signifies an object created below
; which contains all the properties belonging to it. The name of the array
; is unique for each game/special mode even though currently only one of them
; can exist at a time. This way hotswitching can be added later without the
; the program breaking here. It might also avoid issues caused by multiple
; games using the same icon. The IconName is used instead of the Name because it does not
; contain any spaces which would can't be used in an object name.
%CurrentGame%%SpecialMode%RequirementsArray.Insert(IconName)
; Now create the object with its properties. Store the type and icon name,
; then loop to store all the addresses defined with the corresponding
; length and custom code flags. Again add the name of the game and the special
; mode to avoid issues.
%CurrentGame%%SpecialMode%%IconName% := {} ; Create the object
%CurrentGame%%SpecialMode%%IconName%.Insert("Name", Name)
%CurrentGame%%SpecialMode%%IconName%.Insert("Type", Type)
%CurrentGame%%SpecialMode%%IconName%.Insert("TotalRequired", TotalRequired)
; Loop through all the defined addresses, adding them to the object if they are defined.
; Immediately after adding them, clear the address and its length and custom code flags
; in preparation of adding the next requirement once this subroutine is called again.
Loop
{
	if Address%A_Index% = 
		break
	%CurrentGame%%SpecialMode%%IconName%.Insert("Address"A_Index, Address%A_Index%)
	if Address%A_Index%Length =
		Address%A_Index%Length = 4 ; Default length
	%CurrentGame%%SpecialMode%%IconName%.Insert("AddressLength"A_Index, Address%A_Index%Length)
	%CurrentGame%%SpecialMode%%IconName%.Insert("AddressCustomCode"A_Index, Address%A_Index%CustomCode)
	Address%A_Index% = 
	Address%A_Index%Length = 
	Address%A_Index%CustomCode = 
}
; Also reset all the other variables. Only the optional variables need to be reset,
; but resetting all variables makes sure they doesn't cause any issues.
Name =
Type = 
IconName = 
TotalRequired =
return

; ######################################################################################################
; ########################################### SETTINGS WINDOW ##########################################
; ######################################################################################################

/*
Subheadings:

	2ButtonSettings
	4ButtonSave
	4ButtonDiscard/4GuiEscape
	4ButtonRestoreDefault
	4GuiClose
	4GuiContextMenu
	ChangeTextColour
	ChangeBackColour
	ChooseColour
	MaximumRowsText
	MaximumRowsIcons
	RefreshRateUpDown
*/


CreateSettingsWindow:
SettingsTextWidth := 180
gui 4:-MinimizeBox -MaximizeBox +LastFound
; Store the original values in case the settings window is discarded.
TextColourOriginal := TextColour
BackColourOriginal := BackColour
MaximumRowsTextOriginal := MaximumRowsText
MaximumRowsIconsOriginal := MaximumRowsIcons
DecimalPlacesOriginal := DecimalPlaces
/*
RefreshRateOriginal := RefreshRate
*/
RefreshRateFileOutputOriginal := RefreshRateFileOutput
TransparencyOriginal := Transparency
TextSmoothingOriginal := TextSmoothing
OutputWindowBoldTextOriginal := OutputWindowBoldText
ShowDoneIfDoneOriginal := ShowDoneIfDone
ExitConfirmedOriginal := ExitConfirmed
AlwaysOnTopOriginal := AlwaysOnTop

; Add the controls for changing text colour
Gui, 4:Add, Text, y+10 w%SettingsTextWidth% section, Text colour`:
Gui, 4:Add, Button, ys vTextColour gChangeTextColour w60, %TextColour%
; Add the controls for changing background colour
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Background colour`:
Gui, 4:Add, Button, ys vBackColour gChangeBackColour w60, %BackColour%
/*
; Create the structure which contains the custom colours in the colour dialog
; if it doesn't exist already. The custom colours in it need to be saved in the
; settings file.
if CustomColoursStructureExists != 1
{
	VarSetCapacity(CustomColoursStructure, 64, 0)
	CustomColoursStructureExists = 1
}
*/
/*
; Add the controls for changing font
Gui, 4:Add, Text, xs w%SettingsTextWidth% section, Font`:
Gui, 4:Add, Button, ys vFont gChangeFont w60, %Font%
*/
; Add the controls for changing maximum rows text
Gui, 4:Add, Text,xs w%SettingsTextWidth% section vMaximumRowsTextDescription, Text mode maximum rows `(4`-65`)`:
Gui, 4:Add, Slider, ys vMaximumRowsText gMaximumRowsText Range4-65 w120 Tooltip Center Noticks , %MaximumRowsText%
Gui, 4:Add, Text,x+0 vMaximumRowsTextTextControl w12, %MaximumRowsText%
GuiControl, +Buddy2MaximumRowsTextTextControl, MaximumRowsText
; Add the controls for changing maximum rows icons
Gui, 4:Add, Text,xs w%SettingsTextWidth% section vMaximumRowsIconsDescription, Icon mode maximum rows `(3`-30`)`:
Gui, 4:Add, Slider, ys vMaximumRowsIcons gMaximumRowsIcons Range3-30 w120 Tooltip Center Noticks , %MaximumRowsIcons%
Gui, 4:Add, Text,x+0 vMaximumRowsIconsTextControl w12, %MaximumRowsIcons%
GuiControl, +Buddy2MaximumRowsIconsTextControl, MaximumRowsIcons
; Add the controls for changing decimal places
/* 
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Decimal places `(0`-5`)`:
Gui, 4:Add, Slider, ys vDecimalPlaces gDecimalPlaces Range0-5 w120 Tooltip Center Noticks , %DecimalPlaces%
Gui, 4:Add, Text,x+0 vDecimalPlacesTextControl w6, %DecimalPlaces%
GuiControl, +Buddy2DecimalPlacesTextControl, DecimalPlaces 
*/
/*
; Add the controls for changing refresh rate.
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Refresh rate `(100`-10`,000 ms`)`:
Gui, 4:Add, Slider, ys vRefreshRate gRefreshRate Range100-10000 w120 Tooltip Center Noticks, %RefreshRate%
Gui, 4:Add, Text,x+0 vRefreshRateTextControl w30, %RefreshRate%
GuiControl, +Buddy2RefreshRateTextControl, RefreshRate
*/
; Add the controls for changing refresh rate for file output.
RefreshRateFileOutput := Round(RefreshRateFileOutput/1000)
Gui, 4:Add, Text, xs w%SettingsTextWidth% section, Refresh rate file output `(0`-20 sec`)`:
Gui, 4:Add, Slider, ys vRefreshRateFileOutput gRefreshRateFileOutput Range0-20 w120 Tooltip Center Noticks, %RefreshRateFileOutput%
Gui, 4:Add, Text, x+0 vRefreshRateFileOutputTextControl w30, %RefreshRateFileOutput%
GuiControl, +Buddy2RefreshRateFileOutputTextControl, RefreshRateFileOutput
; Add the controls for changing transparency
Transparency := 100-Round(Transparency/2.54)		;Changed from 2.55 to 2.54 to fix it from breaking when at 100%
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Transparency `(0`-100`%`)`:
Gui, 4:Add, Slider, ys vTransparency gTransparency Range0-100 w120 Tooltip Center Noticks , %Transparency%
Gui, 4:Add, Text,x+0 vTransparencyTextControl w28, %Transparency%`%
GuiControl, +Buddy2TransparencyTextControl, Transparency
; Add the controls for changing text smoothing
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Text smoothing`:
Gui, 4:Add, Checkbox, vTextSmoothing ys Checked%TextSmoothing%,
; Add the controls for changing output window bold text
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Output window bold text`:
Gui, 4:Add, Checkbox, vOutputWindowBoldText ys Checked%OutputWindowBoldText%,
; Add the controls for changing show done if done
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Show "DONE" when an item is done`:
Gui, 4:Add, Checkbox, vShowDoneIfDone ys Checked%ShowDoneIfDone%,
; Add the controls for changing exit confirmed
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Skip exit confirmation`:
Gui, 4:Add, Checkbox, vExitConfirmed ys Checked%ExitConfirmed%,
; Add the controls for changing always on top
Gui, 4:Add, Text,xs w%SettingsTextWidth% section, Show checklist always on top`:
Gui, 4:Add, Checkbox, vAlwaysOnTop ys Checked%AlwaysOnTop%,
; Add the buttons for saving, discarding and restoring default settings.
; After all these controls are added, show the window.
gui, 4:Add, Text,xs,
Gui, 4:Add, Button,xs section, Save
Gui, 4:Add, Button,ys, Discard
Gui, 4:Add, Button,ys, Restore Default
gui, 4: Show
return

4ButtonSave:
gui, 4:Submit
; Since everything with the same colour as the background is made transparent
; with transparency on, 'protect' some standard colours.
if (Backcolour = 000000 and Transparency < 255)
	BackColour = 000001
if (Backcolour = F0F0F0 and Transparency < 255)
	BackColour = F1F1F1
/*
if (BackColour = TextColour and Transparency < 255)
{
	BackColour += 1
	BackColour2 := SubStr("00000" . BackColour, -5)
}
*/
; Translate transparency to something the program understands.
Transparency := 255-Round(Transparency*2.54)
; Translate refresh rate from sec to ms for the program.
RefreshRateFileOutput := Round(RefreshRateFileOutput*1000)
; Destroy the settings screen and the welcome screen, save the new settings
; to a file and set the right variables. Then redraw the welcome screen.
gui, 4:Destroy
gui 2:Destroy
gui 5:Destroy
gosub WriteSettingsFile
gosub SetVariablesDependentOnSettings
goto WelcomeScreen
return

; If the settings menu is canceled, undo all the changes made and restore the
; variables to what was saved at the start of initialising the settings window.
; Also destroy the settings window. Since no settings are changed, there is no
; need to destroy and redraw the welcome screen so we just restore it.
4ButtonDiscard:
4GuiEscape:
gui, 4:Destroy
TextColour := TextColourOriginal
BackColour := BackColourOriginal
MaximumRowsText := MaximumRowsTextOriginal
MaximumRowsIcons := MaximumRowsIconsOriginal
DecimalPlaces := DecimalPlacesOriginal
/*
RefreshRate := RefreshRateOriginal
*/
RefreshRateFileOutput := RefreshRateFileOutputOriginal
Transparency := TransparencyOriginal
TextSmoothing := TextSmoothingOriginal
OutputWindowBoldText := OutputWindowBoldTextOriginal
ShowDoneIfDone := ShowDoneIfDoneOriginal
ExitConfirmed := ExitConfirmedOriginal
AlwaysOnTop := AlwaysOnTopOriginal

gui 5:Restore
gui 2:Restore
return

; Restore all the settings to default as defined in the DefaultSettings subroutine.
4ButtonRestoreDefault:
gosub DefaultSettings
Guicontrol,,TextColour,%TextColour%
Guicontrol,,BackColour,%BackColour%
Guicontrol,,MaximumRowsText,%MaximumRowsText%
Guicontrol,,MaximumRowsTextTextControl,%MaximumRowsText%
Guicontrol,,MaximumRowsIcons,%MaximumRowsIcons%
Guicontrol,,MaximumRowsIconsTextControl,%MaximumRowsIcons%
;Guicontrol,,DecimalPlaces,%DecimalPlaces%
;Guicontrol,,DecimalPlacesTextControl,%DecimalPlaces%
/*
Guicontrol,,RefreshRate,%RefreshRate%
Guicontrol,,RefreshRateTextControl,%RefreshRate%
*/
RefreshRateFileOutput := Round(RefreshRateFileOutput/1000)
Guicontrol,,RefreshRateFileOutput,%RefreshRateFileOutput%
Guicontrol,,RefreshRateFileOutputTextControl,%RefreshRateFileOutput%
Transparency := 100-Round(Transparency/2.54)
Guicontrol,,Transparency,%Transparency%
Guicontrol,,TransparencyTextControl,%Transparency%`%
Guicontrol,,TextSmoothing,%TextSmoothing%
Guicontrol,,OutputWindowBoldText,%OutputWindowBoldText%
Guicontrol,,ShowDoneIfDone,%ShowDoneIfDone%
Guicontrol,,ExitConfirmed,%ExitConfirmed%
Guicontrol,,AlwaysOnTop,%AlwaysOnTop%
return

; Close the window.
4GuiClose:
CurrentGUI = 4
exitapp
return

; Add a right click context menu.
4GuiContextMenu:
Menu, OutputRightClick, Add, Restart Script, RestartSequence
Menu, OutputRightClick, Add, Exit, 4GuiClose
Menu, OutputRightClick, Show
return

; Set the colour to change to text colour and start the colour dialog.
ChangeTextColour:
ColourType = Text
gosub ChooseColour
return

; Set the colour to change to background colour and start the colour dialog.
ChangeBackColour:
ColourType = Back
gosub ChooseColour
return

; Initialise the colour dialog, which requires a structure to be set up but
; is otherwise mostly a built-in function.
ChooseColour:
GuiHandle := WinExist()
%ColourType%ColourNew := ChooseColourDialog(%ColourType%Colour, GuiHandle, CustomColoursStructure)
If (%ColourType%ColourNew != "Error")
{
	%ColourType%Colour := %ColourType%ColourNew
	Guicontrol,,%ColourType%Colour,% %ColourType%Colour
}
return

/*
; Call the subroutine which will initialise the font dialog.
ChangeFont:
gosub ChooseFont
return

; Initialise the font dialog, which requires a structure to be set up but
; is otherwise mostly a built-in function.
ChooseFont:
if FontStructureExists = 0
{
	VarSetCapacity(FontStructure, 0x60, 0)
	FontStructureExists = 1
}
GuiHandle := WinExist()
VarSetCapacity(FontDialogStructure, 0x3C, 0)
	NumPut(0x3C, FontDialogStructure, 0x0, "Uint")
	NumPut(GuiHandle, FontDialogStructure, 0x4) ; Will make the font dialog owned by the gui window
;	0x8 is unused
	NumPut(&FontStructure, FontDialogStructure, 0xC) Input/Output structure containing the font attributes
;	0x10 is the output for the font size
;	0x14 contains flags to set for creating the font dialog
;	0x18 contains a RGB macro if that flag is set in 0x14
;	0x1C is lCustData, not sure what this does
;	0x20 is a handle to some thing that handles messages if that flag is set in 0x14
;	0x24 is the name of the custom template for the font dialog in 0x28
; 	0x28 is the handle to the custom template if that flag is set in 0x14
;	0x2C is the input/output for a font style combo box if that flag is set in 0x14
;	0x30 is the output for the font type (e.g. bold, italic, etc)
;	0x34 is the minimum font size if that flag is set in 0x14
;	0x38 is the maximum font size if that flag is set in 0x14
ChooseFontErrorLevel := DllCall("Comdlg32\ChooseFont", "Ptr", &FontDialogStructure)
if ChooseFontErrorLevel != 0
{
	FontSize := NumGet(FontDialogStructure, 0x10, "Int") / 10
	FontType := NumGet(FontDialogStructure, 0x30, "Int")
	msgbox ok %FontSize% %FontType%
}
return
*/

; Update the text next to the maximum text rows slider.
MaximumRowsText:
Guicontrol,,MaximumRowsTextTextControl,%MaximumRowsText%
return

; Update the text next to the maximum icons rows slider.
MaximumRowsIcons:
Guicontrol,,MaximumRowsIconsTextControl,%MaximumRowsIcons%
return

; Update the text next to the decimal places slider.
DecimalPlaces:
Guicontrol,,DecimalPlacesTextControl,%DecimalPlaces%
return

/*
; Update the text next to the refresh rate slider.
RefreshRate:
Guicontrol,,RefreshRateTextControl,%RefreshRate%
return
*/

; Update the text next to the refresh rate for file output slider.
RefreshRateFileOutput:
Guicontrol,,RefreshRateFileOutputTextControl,%RefreshRateFileOutput%
return

; Update the text next to the transparency slider.
Transparency:
Guicontrol,,TransparencyTextControl,%Transparency%`%
return



; ######################################################################################################
; ########################################### OUTPUT WINDOW ############################################
; ######################################################################################################

/*
Subheadings:

	OutputWindow
	GuiClose/GuiEscape
	GuiContextMenu
	MoveWindow
	WM_LBUTTONDOWN
	TextPopulateListView
	IconsPopulateListView
	ButtonChangeOutputType
*/

OutputWindow:
Gui 1:-MinimizeBox -MaximizeBox +LastFound
Gui, 1:Default
OutputFont = 0
if AlwaysOnTop = 1
	Winset, AlwaysOnTop, On
If (Transparency < 255 or OutputWindowBoldText = 1)
{
	Gui, 1:Font, w1000
	OutputFont = w1000
}
If (Transparency < 255 or TextSmoothing = 0)
{
	Gui, 1:Font, Q3
	OutputFont = Q3
}
Gui, 1:Add, Button, , Change Output Type
ListViewNumber = 1
RowAmount = 0
MaximumRows := MaximumRows%OutputType%
MaxValueLengthInPixels%ListViewNumber% = 0

TextListViewWidth%ListViewNumber% := 0

IconListViewWidth%ListViewNumber% := 0
IconListViewWidthWithFloat := IconListViewWidth%ListViewNumber%+5*DecimalPlaces

SetFormat, Float, 0.20
LastCol := Ceil(%CurrentGame%%SpecialMode%RequirementsArray.MaxIndex()/MaximumRows)
SetFormat, Float, 0.%DecimalPlaces%
LongestString%ListViewNumber% := 0

For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	Name := %CurrentGame%%SpecialMode%%IconName%.Name
	RowAmount += 1
	If RowAmount > %MaximumRows%
	{
		RowAmount = 1
		ListViewNumber += 1
	}
	TempStringWidth := StringWidth(Name,,%OutputFont%)
	If( TempStringWidth > LongestString%ListViewNumber% )
		LongestString%ListViewNumber% := TempStringWidth
}

ListViewNumber := 1
RowAmount := 0

If (OutputType = "Text")
{
	Gui, 1:Add, ListView, c%TextColour% vRequirementsListView%ListViewNumber% Background%BackColour% gMoveWindow w%TextListViewWidth%%ListViewNumber% Count20 -Multi -E0x200 section -hdr +0x2000, Name|Value|Required
}
Else If (OutputType = "Icons")
{
	Gui, 1:Add, ListView, c%TextColour% vRequirementsListView%ListViewNumber% Background%BackColour% gMoveWindow w%IconListViewWidth%%ListViewNumber% Count15 -Multi -E0x200 section +Tile -LV0x20 +LV0x1 +0x2000 -0x8, Value
	ImageListID := IL_Create(20, 1, 1)
	LV_SetImageList(ImageListID, 0) ; 0 specifies large icons
	
}
Else
{
	msgbox, Error`: Unknown output type`.
	ExitConfirmed = 1
	exitapp
}
;GuiControl, Move, RequirementsListView%ListViewNumber%, w250

; For each requirement in the requirements array, add it to the list view.
; At the start of every requirement, check if the list view exceeds the maximum
; length and create a new one in necessary.
For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	Name := %CurrentGame%%SpecialMode%%IconName%.Name
	Type := %CurrentGame%%SpecialMode%%IconName%.Type
	TotalRequired := %CurrentGame%%SpecialMode%%IconName%.TotalRequired
	RowAmount += 1

	If RowAmount > %MaximumRows%
	{
		RowAmount = 1
		TotalRows := LV_GetCount()
		; For icons mode: each icon is 32x32 pixels with 4 pixels vertical padding between icons.
		; For text mode: each text has a height of 11 pixels with 4 pixels vertical padding at the top, 2 pixels at the bottom.
		ListViewHeight := TotalRows*((OutputType = "Text") ? 17 : 36)
		Guicontrol, Move, RequirementsListView%ListViewNumber%, h%ListViewHeight%
		; Update the column width and then update the list view width based on the column width.
		; The reason the column width needs to be updated is because of the variable length of
		; the requirement name, so this is only necessary in text mode (which is good because the
		; code wouldn't work for icons since it doesn't take the icon size into account).
		If (OutputType = "Text")
		{
			;LV_ModifyCol() ; Set the width for the columns
			ListViewTargetWidth = 10
			If FloatInListView%ListViewNumber% = 1
				ListViewTargetWidth += Round(5.6*DecimalPlaces)
			Loop % LV_GetCount("Column")
			{
				SendMessage, 4125, A_Index-1, 0, SysListView32%ListViewNumber%  ; 4125 is LVM_GETCOLUMNWIDTH.
				ListViewTargetWidth += %ErrorLevel%
			}
			;GuiControl, Move, RequirementsListView%ListViewNumber%, w%ListViewTargetWidth%
		}
		; We need to determine the X for the new list view manually, since we potentially changed
		; its size. The gui add function doesn't keep track of that by itself so it would put the
		; new listview based on the old width and the two list views would overlap or have too much padding.
		ControlGetPos,PreviousListViewX,,PreviousListViewWidth,,SysListView32%ListViewNumber%,,,
		NewX := PreviousListViewX+PreviousListViewWidth
		ListViewNumber++
		MaxValueLengthInPixels%ListViewNumber% = 0
		If (OutputType = "Text")
		{
			Gui, 1:Add, ListView, c%TextColour% vRequirementsListView%ListViewNumber% Background%BackColour% gMoveWindow w%TextListViewWidth%%ListViewNumber%  Count20 -Multi -E0x200 x%NewX% ys -Hdr +0x2000 ,Name|Value|Required
		}
		Else If (OutputType = "Icons")
		{
			Gui, 1:Add, ListView, c%TextColour% vRequirementsListView%ListViewNumber% Background%BackColour% gMoveWindow w%IconListViewWidth%%ListViewNumber% Count15 -Multi -E0x200 x%NewX% ys +Tile -LV0x20 +LV0x1 +0x2000 -0x8,Value
			LV_SetImageList(ImageListID, 0) ; 0 specifies large icons
		}
	}
	
	

	If (OutputType = "Text")
	{
		; Create the entry in the listview. Check if TotalRequired is defined, if not leave it out.
		LV_Add("", Name, 0, ((TotalRequired != "") ? ("`/"TotalRequired) : null ))
		;If Type = Float
			;FloatInListView%ListViewNumber% = 1
			
		If(StringWidth(TotalRequired,,OutputFont) > MaxValueLengthInPixels%ListViewNumber%)
			MaxValueLengthInPixels%ListViewNumber% := StringWidth(TotalRequired,,OutputFont)

		If(StringWidth(name,,OutputFont) > TextListViewWidth%ListViewNumber%)
			TextListViewWidth%ListViewNumber% := StringWidth(name,,OutputFont)
		
		TextViewWidth := 7 + TextListViewWidth%ListViewNumber% + MaxValueLengthInPixels%ListViewNumber% *2
		If(OutputWindowBoldText == 1 && LastCol != ListViewNumber)
			TextViewWidth += 20
		If ( Transparency < 255 && LastCol != ListViewNumber)
			TextViewWidth += 20
		If(LastCol != ListViewNumber)
			TextViewWidth += 6
		GuiControl, Move, RequirementsListView%ListViewNumber%, w%TextViewWidth%
		Dot := .
		DotsToAdd := Floor((LongestString%ListViewNumber% - StringWidth(name,,OutputFont)) / StringWidth(Dot,,OutputFont))
		
		Loop, %DotsToAdd%{
			LV_GetText(TempText, RowAmount)
			LV_Modify(RowAmount, "col1", TempText Dot)
			
		}
		
		ColWidth := LongestString%ListViewNumber%
		TextOffset := -40
		If(OutputWindowBoldText = 1 or Transparency < 255)
			TextOffset := -20
		LV_ModifyCol(1, ColWidth + TextOffset)
		
	}
	Else If (OutputType = "Icons")
	{
		IconNumber := IconArray[IconName]
		IconIndex := IL_Add(ImageListID, ImageFilename, IconNumber)
		; Create the entry in the listview. Check if TotalRequired is defined, if not leave it out.
		LV_Add("Icon" . IconIndex, ((TotalRequired != "") ? ("0`/"TotalRequired) : 0 ))
		
		MaxSize := 44 + (StringWidth(TotalRequired,,OutputFont)*2)
		
		if (MaxSize > IconListViewWidth%ListViewNumber% )
		{
			IconListViewWidth%ListViewNumber% := MaxSize
		}
		
		if Type = Float
		{
			IconListViewWidth%ListViewNumber% := IconListViewWidth%ListViewNumber% + 6 * DecimalPlaces
		}
		
		IconListViewWidth := IconListViewWidth%ListViewNumber%
		
		GuiControl, Move, RequirementsListView%ListViewNumber%, w%IconListViewWidth%
		
	}
}

GuiWidth := 5
If(OutputType = "Icons"){
	GuiWidth := 12 + (ListViewNumber*2)
}
Loop, %ListViewNumber%{ 
	ControlGetPos,,,RequirementsListViewWidth,,SysListView32%A_Index%
	
	GuiWidth += RequirementsListViewWidth
}

TotalRows := LV_GetCount()
gui, 1:Color, %BackColour%, %BackColour%
if Transparency < 255
{
	WinSet, TransColor, %BackColour%
	OutputWindowID := WinExist()
	OnMessage(0x0047, "WM_OUTPUTWINDOWPOSCHANGED")
}
else
{
	gui 1:-Caption
	OnMessage(0x201, "WM_LBUTTONDOWN")
}

 If (OutputType = "Text")
{
	;LV_ModifyCol(5) ; Set the width for the columns
	
/* 	If FloatInListView%ListViewNumber% = 1
		ListViewTargetWidth += Round(5.6*DecimalPlaces)
	ListViewTargetWidth = 24
	Loop % LV_GetCount("Column")
	{
		SendMessage, 4125, A_Index-1, 0, SysListView32%ListViewNumber%  ; 4125 is LVM_GETCOLUMNWIDTH.
		ListViewTargetWidth += %ErrorLevel%
	} 
*/
	;GuiControl, Move, RequirementsListView%ListViewNumber%, w%ListViewTargetWidth%
} 



; For text mode: each text has a height of 11 pixels with 4 pixels vertical padding at the top, 2 pixels at the bottom, for a total of 17.
; For icons mode: each icon is 32x32 pixels with 4 pixels vertical padding between icons, for a total of 36.

ListViewHeight := TotalRows*((OutputType = "Text") ? 17 : 36)
Guicontrol, Move, RequirementsListView%ListViewNumber%, h%ListViewHeight%

; Set the height of the window in pixels. We can describe the window in three parts:
; From the top of the window (so including the button) to the list view, which is 35 pixels.
; The height of the list view (the first list view is always the longest so use that one).
; Padding at the bottom, which can be chosen as whatever. We will use 5 here.
ControlGetPos,,,,RequirementsListViewHeight,SysListView321,,,
GuiHeight := 35+RequirementsListViewHeight+5
; Check if this is the first time the window is shown or if it is redrawn after changing the output type.
; If it is being redrawn, use the saved position of the original window to draw it at the same position.
If OutputChanged = 1
{
	gui, 1:Show, h%GuiHeight% x%OutputWindowX% y%OutputWindowY% w%GuiWidth%
}
else
{
	gui, 1:Show, h%GuiHeight% w%GuiWidth%
}
; If any level of transparency is in play, the background window is made entirely transparent and some
; preparations are made for faking a semi-transparent background. After the gui is rendered, a second
; window is created which is locked to the first. It consists of just a background which is semi-
; transparent. (Within one window, it is possible to make either the background completely transparent,
; or the entire window, including controls, semi-transparent. This is the only decent solution I have
; found. It does have some issues (see readme) but nothing major.) The background window is set to be
; the owner of the real window, to avoid it being selectable which would push the background over the
; other window. Even if the background is completely transparent, the background window is still created,
; because otherwise clicks would fall through (this only happens if a specific colour is made transparent
; as is the case for the real window, it doesn't happen if the entire window is made completely transparent
; as is the case for the background window.)
If Transparency < 255
{
	WinGetPos,,, OutputWindowNoCaptionWidth, OutputWindowNoCaptionHeight
	WinGetPos, OutputWindowX, OutputWindowY, OutputWindowWidth, OutputWindowHeight
	BackgroundX := OutputWindowX+OutputWindowWidth-OutputWindowNoCaptionWidth
	BackgroundY := OutputWindowY+OutputWindowHeight-OutputWindowNoCaptionHeight
	gui 6: +LastFound
	gui, 6:Color, %BackColour%, %BackColour%
	Winset, Transparent, %Transparency%
	if AlwaysOnTop = 1
		Winset, AlwaysOnTop, On
	gui 6: -Caption
	Gui, 6: Show, x%BackgroundX% y%BackgroundY% w%OutputWindowNoCaptionWidth% h%OutputWindowNoCaptionHeight% NoActivate
	gui 1: +Owner6
	gui, 1:Show, h%GuiHeight% w%GuiWidth%
	if AlwaysOnTop = 1
		Winset, AlwaysOnTop, On
}


; Proceed to the MainScript, which is where the output is updated. Since we don't want to start the
; MainScript after changing the output (since it will be running already), we check for that. If the
; MainScript would be launched every time, we would quickly reach the maximum number of simultaneous
; threads causing the whole program to become unresponsive. Not to mention the memory leaking from having
; a lot of threads running.
if OutputChanged != 1
	goto MainScript
else
	exit


GuiClose:
GuiEscape:
CurrentGUI = 1
exitapp
return

; The following items are added to the right click menu.
GuiContextMenu:
Menu, OutputRightClick, Add, Restart Script, RestartSequence
Menu, OutputRightClick, Add, Exit, GuiClose
Menu, OutputRightClick, Show
return

; This function is triggered if the output window is moved, in order to move the 'fake' semi-transparent background with it.
WM_OUTPUTWINDOWPOSCHANGED()
{
	global
	gui 1: +LastFound
	WinGetPos,OutputWindowX, OutputWindowY,
	gui 6: +LastFound
	BackgroundX := OutputWindowX+(OutputWindowWidth-OutputWindowNoCaptionWidth)
	BackgroundY := OutputWindowY+(OutputWindowHeight-OutputWindowNoCaptionHeight)
	WinMove, BackgroundX, BackgroundY
}

; When the user clicks on any of the controls and moves the mouse, this will drag the window with it.
MoveWindow: ; For clicking on controls
if Transparency = 255
	PostMessage, 0xA1, 2,,, A     ; Drag window on click
return

; When the user clicks in the window (but not on the controls) and moves the mouse, this will drag the window with it.
WM_LBUTTONDOWN(wParam, lParam) ; For clicking anywhere else in the window
{
	PostMessage, 0xA1, 2,,, A     ; Drag window on click
}

; Pressing the "Change output type" button will alternate the output between text and icons.
; To accomplish this, it changes the OutputType and then destroys and recreates the output window.
; The position of the window is also saved to be able to draw the recreated output window at the
; same position. Since the image list is potentially shared between multiple list views, it won't
; be destroyed automatically so this is done explicitly.
ButtonChangeOutputType:
If (OutputType = "Text")
	OutputType = Icons
Else If (OutputType = "Icons")
	OutputType = Text
WinGetPos,OutputWindowX, OutputWindowY,
gui 1:Destroy
gui 6:Destroy
IL_Destroy(ImageListID)
OutputChanged = 1
OutputChangedDup = 1
goto OutputWindow
return


; ######################################################################################################
; ############################################ FILE OUTPUT #############################################
; ######################################################################################################

FileOutput:
; MaximumRows := MaximumRows%OutputType% Not supported for now.
FileOutputName := ScriptNameNoExt A_Space "Output.txt"
FileOutputNameTemp := "Temp" A_Space FileOutputName ; Used to reduce the downtime when updating the file as far as possible.
if FileExist(FileOutputName)
	FileDelete, %FileOutputName%
For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	Name := %CurrentGame%%SpecialMode%%IconName%.Name
	TotalRequired := %CurrentGame%%SpecialMode%%IconName%.TotalRequired
	; Create the entry in the listview. Check if TotalRequired is defined, if not leave it out.
	TextEntry := Name A_Space "0" ((TotalRequired != "") ? ("`/"TotalRequired) : "" ) "`n"
	FileAppend, %TextEntry%, %FileOutputName%
}
goto MainScript

; ######################################################################################################
; ########################################### UPDATE OUTPUT ############################################
; ######################################################################################################

/*
Subheadings:

	MainScript
	UpdateOutputCode
	ResetOutputCode
*/


MainScript:

; Get the window class and name of the selected game from the array.
WindowClass := "Gamebryo Application"
WindowName := "Bully"
; Wait until the game window is started. Check both the window class and window title to avoid false positives.
WinWait ahk_class %WindowClass%
WinGetTitle, CurrentWindowName

WinGet, PID, PID
ErrorLevel := Memory(1, PID)
If ErrorLevel != 0
{
	If A_IsAdmin = 0
	{
		msgbox Error accessing the game. `nThe program will now try to restart with admin privileges.
		Run *RunAs "%A_ScriptFullPath%"
	}
	Else
	{
		msgbox Error accessing the game. `nThe program cannot continue operating.`n%Error%.
		Error := GetLastErrorMessage()
	}
	ExitConfirmed = 1
	ExitApp
}
; Check if the game is started (which will set the ErrorLevel to !=0),
; then check which version of the current game is used and which offset to use for memory addresses.
Process, Exist, %PID%
If ErrorLevel == 0
	goto MainScript
; Show a tray tip to the user explaining what the program will do next.
Traytip, %ScriptNameNoExt%, The program is now running in the background and will update automatically,20,
; Get which address to check if the game is still running.
GameRunningAddress := 0x00400000
; Reset all the values of the requirements to be sure to start with a clean slate.
; (Why is this necessary?)
ListViewNumber = 1
RowNumber = 0
gosub ResetOutput
; This While-loop will remain active as long as the game is running. The code in it updates the output.
; The loop checks if the output type is changed because it needs to realize that in order to keep
; updating the output.
While Memory(3, GameRunningAddress, 1) != "Fail"
{
	If (OutputType = "File")
		gosub UpdateOutputFile
	Else
	{
		ListViewNumber = 1
		Gui, 1:ListView, RequirementsListView%ListViewNumber%
		RowNumber = 0
		gosub UpdateOutput
	}
	if OutputChangedDup = 1
	{
		OutputChangedDup = 0
		break
	}
	sleep %RefreshRate%
}
; If the while-loop breaks (meaning the game is no longer running), reset the
; output for each requirement. After that return to the start of the 'main script'
; where the program will once again wait until the game is started.
If (OutputType = "File")
	gosub ResetOutputFile
Else
{
	ListViewNumber = 1
	Gui, 1:ListView, RequirementsListView%ListViewNumber%
	RowNumber = 0
	gosub ResetOutput
}
goto MainScript


; Code which updates the output.
UpdateOutput:
For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	TotalRequired := %CurrentGame%%SpecialMode%%IconName%.TotalRequired
	Type := %CurrentGame%%SpecialMode%%IconName%.Type
	; The decimal places setting is enforced
	; (Why is this necessary?)
	SetFormat, Float, 0.%DecimalPlaces%
	; sleep 100
	; The current value is reset to stop information from carrying over from the previous requirement.
	CurrentValue = 0
	; Then the row (and listview if required) which will be updated is selected.
	RowNumber += 1
	
	
	
	if RowNumber > %MaximumRows%
	{
		RowNumber = 1
		ListViewNumber++
		Gui, 1:ListView, RequirementsListView%ListViewNumber%
	}
	; Loop to read all the memory addresses belonging to the current requirement.
	; Get the address, length and customcode flags from the requirements array.
	; If type is set, the read value can be converted to a float and custom code
	; located in the requirement list can be executed. At the end of the loop,
	; The variable 'CurrentValue' contains the finalized value of the requirement.
	Loop
	{
		ReadAddress := %CurrentGame%%SpecialMode%%IconName%["Address"A_Index]
		if ReadAddress = 
			break
		ReadLength := %CurrentGame%%SpecialMode%%IconName%["AddressLength"A_Index]
		MemoryValue := Memory(3, ReadAddress, ReadLength)
		if Type = Float
			MemoryValue := HexToFloat(MemoryValue)
		if (%CurrentGame%%SpecialMode%%IconName%["AddressCustomCode"A_Index] = 1)
			gosub %IconName%Address%A_Index%CustomCode
		CurrentValue += %MemoryValue%
	}
	; Only update the output if the value found in this cycle is not the same as
	; the value found in the last cycle.
	if (CurrentValue != %IconName%ValueOld)
	{
		/*
		; Test to remove each item once it's done.
		; Issues atm:
		; No way to re-add items when reloading a save/starting a new game.
		; Items after a deleted entry are no longer updated.
		if (CurrentValue >= TotalRequired AND TotalRequired != "")
		{
			LV_Delete(RowNumber)
			RequirementsArrayClone := %CurrentGame%%SpecialMode%RequirementsArray.Clone()
			%CurrentGame%%SpecialMode%RequirementsArray.Remove(Index)
			ItemsRemoved = 1
			continue
		}
		*/
		; Store the value found this cycle to compare the next cycle against.
		%IconName%ValueOld := CurrentValue
		; When the requirement is done and the 'show DONE if done' option is selected,
		; write 'DONE' to the requirement and skip the rest of this cycle.
		if (CurrentValue >= TotalRequired AND TotalRequired != "" AND ShowDoneIfDone = 1)
		{
			if (OutputType = "Icons")
				LV_Modify(RowNumber,"Col1", "DONE")
			Else If (OutputType = "Text")
			{
				LV_Modify(RowNumber,"Col2", "")
				LV_Modify(RowNumber,"Col3", "DONE")
				LV_ModifyCol(2)
				LV_ModifyCol(3)
			}
			continue
		}
		; If a total required is defined, check if the value is higher and
		; enforce it as a maximum in necessary.
		if (CurrentValue > TotalRequired AND TotalRequired != "")
			CurrentValue := TotalRequired
		; Update the output, with slightly different code depending on the
		; output type and if a total required has been defined.
		If (OutputType = "Icons")
		{
			UpdatedColumnContents := ((TotalRequired != "") ? (CurrentValue "`/" TotalRequired) : (CurrentValue) )
			LV_Modify(RowNumber,"Col1", UpdatedColumnContents)
			UpdatedValueLengthChar := StrLen(UpdatedColumnContents)
		}
		Else If (OutputType = "Text")
		{
			LV_Modify(RowNumber,"Col2", CurrentValue)
			If(TotalRequired != "")
				LV_Modify(RowNumber,"Col3", "/"TotalRequired)	;Restores TotalRequired if was done but save was loaded
			UpdatedValueLengthChar := StrLen(CurrentValue)
		}
		; Check the length of the new value and make the output wider if it's too small.
		UpdatedValueLengthInPixels := StringWidth(CurrentValue, ,OutputFont) + 15
		if (UpdatedValueLengthInPixels > MaxValueLengthInPixels%ListViewNumber%)
		{
			If (OutputType = "Text")
				TextOffset := -10
				If (OutputWindowBoldText = 1 or Transparency < 255)
					TextOffset := -5
				LV_ModifyCol(2,UpdatedValueLengthInPixels+TextOffset)
			
			/*
			Else If (OutputType = "Icons")
			{
				UpdatedValueLengthInPixels := UpdatedValueLengthInPixels+32
				GuiControlGet, RequirementsListView%ListViewNumber%Pos, Pos, RequirementsListView%ListViewNumber%
				Guicontrol, Move, RequirementsListView%ListViewNumber%, w%UpdatedValueLengthInPixels%
				LV_ModifyCol(1,UpdatedValueLengthInPixels)
				UpdatedValueLengthInPixels := UpdatedValueLengthInPixels-32
			}
			*/
			MaxValueLengthInPixels%ListViewNumber% := UpdatedValueLengthInPixels
		}
	}
}
return

; For each requirement in the requirements array, set the value in the listview(s) back to
; 0 or 0/TotalRequired, whichever applies. At the start of every requirement, check if the
; current row exceeds the maximum length and jump to the next list view in necessary.
ResetOutput:
For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	TotalRequired := %CurrentGame%%SpecialMode%%IconName%.TotalRequired
	RowNumber += 1
	%IconName%ValueOld = 0
	If RowNumber > %MaximumRows%
	{
		RowNumber = 1
		ListViewNumber += 1
		Gui, 1:ListView, RequirementsListView%ListViewNumber%
	}
	If (OutputType = "Icons")
		LV_Modify(RowNumber,"Col1", ((TotalRequired != "") ? ("0`/"TotalRequired) : 0 ))
	Else If (OutputType = "Text")
		LV_Modify(RowNumber,"Col2", 0)
}
return

UpdateOutputFile:
if FileExist(FileOutputNameTemp)
	FileDelete, %FileOutputNameTemp%
For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	Name := %CurrentGame%%SpecialMode%%IconName%.Name
	TotalRequired := %CurrentGame%%SpecialMode%%IconName%.TotalRequired
	Type := %CurrentGame%%SpecialMode%%IconName%.Type
	; The current value is reset to stop information from carrying over from the previous requirement.
	CurrentValue = 0
	; Loop to read all the memory addresses belonging to the current requirement.
	; Get the address, length and customcode flags from the requirements array.
	; If type is set, the read value can be converted to a float and custom code
	; located in the requirement list can be executed. At the end of the loop,
	; The variable 'CurrentValue' contains the finalized value of the requirement.
	Loop
	{
		ReadAddress := %CurrentGame%%SpecialMode%%IconName%["Address"A_Index]
		if ReadAddress = 
			break
		ReadLength := %CurrentGame%%SpecialMode%%IconName%["AddressLength"A_Index]
		MemoryValue := Memory(3, ReadAddress, ReadLength)
		if Type = Float
			MemoryValue := HexToFloat(MemoryValue)
		if (%CurrentGame%%SpecialMode%%IconName%["AddressCustomCode"A_Index] = 1)
			gosub %IconName%Address%A_Index%CustomCode
		CurrentValue += %MemoryValue%
	}
	; When the requirement is done and the 'show DONE if done' option is selected,
	; write 'DONE' to the requirement. Else, enforce the TotalRequired as a maximum for CurrentValue
	if (CurrentValue >= TotalRequired AND TotalRequired != "")
	{
		if (ShowDoneIfDone = 1)
			TextEntry := Name A_Space "DONE" "`n"
		Else
			TextEntry := Name A_Space TotalRequired "`/" TotalRequired "`n"
	}
	Else
		TextEntry := Name A_Space CurrentValue  ((TotalRequired != "") ? ("`/"TotalRequired) : "" ) "`n"
	FileAppend, %TextEntry%, %FileOutputNameTemp%
}
if FileExist(FileOutputNameTemp)
	FileMove, %FileOutputNameTemp%, %FileOutputName%, 1 ; Move the temp file to the original file output, overwriting the old file.
Else
	msgbox Error`: Updated output file unavailable`.
sleep %RefreshRateFileOutput% ; Updating the file is quite a resource hog, so we don't want to do it too often.
return


ResetOutputFile:
if FileExist(FileOutputName)
	FileDelete, %FileOutputName%
if FileExist(FileOutputNameTemp)
	FileDelete, %FileOutputNameTemp%
For Index, IconName in %CurrentGame%%SpecialMode%RequirementsArray
{
	Name := %CurrentGame%%SpecialMode%%IconName%.Name
	TotalRequired := %CurrentGame%%SpecialMode%%IconName%.TotalRequired
	; Create the entry in the listview. Check if TotalRequired is defined, if not leave it out.
	TextEntry := Name " 0" ((TotalRequired != "") ? ("`/"TotalRequired) : "" )
	FileAppend, %TextEntry%, %FileOutputName%
}
return


; ######################################################################################################
; ########################################### RESTART/EXIT SEQUENCE ####################################
; ######################################################################################################

/*
Subheadings:

	RestartSequence
	ExitSequence
	3ButtonYes
	3ButtonNo/3GuiClose/3GuiEscape
*/

; Restart the program.
RestartSequence:
ReloadingScript = 1
reload
sleep 100
return


; If the exit is already confirmed or the script is being reloaded, do that immediately.
; If not, disable the currently active window (which prevents it from being activated)
; and instead create a window asking the user to confirm the exit. This window does not
; follow the settings to make sure it is always readable, even if the user screwed up the
; settings. The only setting which is used here is the text smoothing, since it can't
; mess up the window so far that it is no longer readable.
ExitSequence:
If (ExitConfirmed = 1 or ReloadingScript = 1)
{
	; Delete the icons library if the program is compiled to keep things neat.
	if A_IsCompiled = 1
		FileDelete, Icons.icl
	; If the program is exited mid update loop in file mode, we need to remove the temp file here.
	if FileExist(FileOutputNameTemp)
		FileDelete, %FileOutputNameTemp%
	Exitapp
}
else
{
	if CurrentGUI != 
		gui %CurrentGUI%:+disabled
	gui 3:-MinimizeBox -MaximizeBox +owner%CurrentGUI% +LastFound
	Winset, AlwaysOnTop, On
	if TextSmoothing = 0
		gui, 3:Font, Q3
	Gui, 3:Add, Text,, Are you sure you want to exit the program?
	Gui, 3:Add, Button, Default section, Yes
	Gui, 3:Add, Button, ys, No
	gui, 3:show,,
	return
}

; If the user confirms the exit, exit.
3ButtonYes:
ExitConfirmed = 1
ExitApp

; If the user cancels the exit, restore the previous window and destroy the exit confirmation window.
3ButtonNo:
3GuiClose:
3GuiEscape:
if CurrentGUI != 
	gui %CurrentGUI%:-disabled
gui, 3:destroy
return

; ######################################################################################################

;Loha : Bully section handled by LohaTronsRS, I did mostly use zoton2's methods here as it was more applicable

/*
Subheadings:

	BullyPercentageOnly
	BullyRequirements
	Custom Codes:
		BProgressAddress1CustomCode
		BullyMissionsAddress1CustomCode
		BullyCloAddress1CustomCode
		BullyOnFootAddress1CustomCode
		BullyChemAddress1CustomCode
		BullyTombAddress1CustomCode
		BullyPumpAddress1CustomCode
		BullyEnglishAddress1CustomCode
		BullyArtAddress1CustomCode
		BullyGymAddress1CustomCode
		BullyBiologyAddress1CustomCode
		BullyMusicAddress1CustomCode
		BullyShopAddress1CustomCode
		BullyPhotoClassAddress1CustomCode
		BullyGeoAddress1CustomCode
		BullyMathAddress1CustomCode
		BullyBoxingAddress1CustomCode
		BullyValeBRAddress1CustomCode
		BullyTownBRAddress1CustomCode
		BullyCoveBRAddress1CustomCode
		BullyAcaBRAddress1CustomCode
		BullyGPRAddress1CustomCode
		BullyGOStreetAddress1CustomCode
		BullyKeepUpAddress1CustomCode
		BullyPenShootAddress1CustomCode
		BullyPRouteAddress1CustomCode
		BullyLMParkAddress1CustomCode
		BullyLMHouseAddress1CustomCode
		BullyErrandsAddress1CustomCode
		BullyFireAlarmAddress1CustomCode
		BullyOnBikeAddress1CustomCode
		BullyProjHitsAddress1CustomCode
		BullyCarGamesAddress1CustomCode
		BullyCarnRidesAddress1CustomCode
		BullyMonkeyHSAddress1CustomCode
		BullySumoHSAddress1CustomCode
		BullyNutHSAddress1CustomCode
		BullyLockerCloAddress1CustomCode
		BullyDunceAddress1CustomCode
		BullyPirateAddress1CustomCode
		BullyGnomeAddress1CustomCode
		BullyRadioAddress1CustomCode
		BullyRubberAddress1CustomCode
		BullyCardsAddress1CustomCode
		BullyYearbookAddress1CustomCode
*/


;Loha : Im aware i could have probably done some of these without custom coding , but since i need all values 
;		for percentage anyway, might as well just reuse them as custom codes
BullyPercentageOnlyRequirements:

Name = Percentage Completed
Type = Float
IconName = BProgress
Address1 := 0x020C3304
gosub %CurrentLoopCode%

Name = Simulated Percentage
Type = Float
IconName = BSProgress
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%
return

BullyRequirements:

Name = Percentage Completed
Type = Float
IconName = BProgress
Address1 := 0x020C3304
gosub %CurrentLoopCode%

Name = Simulated Percentage
Type = Float
IconName = BSProgress
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Missions
IconName = BullyMissions
TotalRequired = 76
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Errands
IconName = BullyErrands
TotalRequired = 50
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Chemistry
IconName = BullyChem
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = English
IconName = BullyEnglish
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Art
IconName = BullyArt
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Gym
IconName = BullyGym
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Biology
IconName = BullyBiology
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Music
IconName = BullyMusic
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Shop
IconName = BullyShop
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Photography
IconName = BullyPhotoClass
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Geography
IconName = BullyGeo
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Math
IconName = BullyMath
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Dunce Cap
IconName = BullyDunce
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Yearbook
IconName = BullyYearbook
TotalRequired = 61
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Boxing
IconName = BullyBoxing
TotalRequired = 4
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Keep-Ups
IconName = BullyKeepUp
TotalRequired = 1
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Penalty Shootout
IconName = BullyPenShoot
TotalRequired = 1
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Paper Route (Intro/10)
IconName = BullyPRoute
TotalRequired = 2
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Lawn Mowing: Park
IconName = BullyLMPark
TotalRequired = 3
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Lawn Mowing: House
IconName = BullyLMHouse
TotalRequired = 3
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Vale Bike races
IconName = BullyValeBR
TotalRequired = 7
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Town Bike races
IconName = BullyTownBR
TotalRequired = 4
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = New Coventry Bike races
IconName = BullyCoveBR
TotalRequired = 2
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Academy Bike race
IconName = BullyAcaBR
TotalRequired = 1
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Grand Prix Races 
IconName = BullyGPR
TotalRequired = 5
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Street Races 
IconName = BullyGOStreet
TotalRequired = 3
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Carnival Games
IconName = BullyCarGames
TotalRequired = 4
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Carnival Rides
IconName = BullyCarnRides
TotalRequired = 3
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Clothing Items Acquired 
IconName = BullyClo
TotalRequired = 334
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Locker masks
IconName = BullyLockerClo
TotalRequired = 2
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Secret Pirate 
IconName = BullyPirate
TotalRequired = 1
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%


Name = Fire Alarms Pulled 
IconName = BullyFireAlarm
TotalRequired = 20
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Gnomes
IconName = BullyGnome
TotalRequired = 25
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Radio Transistors
IconName = BullyRadio
TotalRequired = 6
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Rubber Bands
IconName = BullyRubber
TotalRequired = 75
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = "G&G" Collectible Cards
IconName = BullyCards
TotalRequired = 40
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Pumpkins
IconName = BullyPump
TotalRequired = 27
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Tombstones
IconName = BullyTomb
TotalRequired = 19
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Sodas Purchased 
IconName = BullySodaPurchase
TotalRequired = 100
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Sodas Drank
IconName = BullySoda
TotalRequired = 500
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Travelled on Foot
Type = Float
IconName = BullyOnFoot
TotalRequired = 100
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Travelled on Bike
Type = Float
IconName = BullyOnBike
TotalRequired = 100
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Projectile Hits
IconName = BullyProjHits
TotalRequired = 1000
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Monkey Fling HiScore
IconName = BullyMonkeyHS
TotalRequired = 333
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = ConSumo HiScore
IconName = BullySumoHS
TotalRequired = 1010
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

Name = Nut Shots HiScore
IconName = BullyNutHS
TotalRequired = 69900
Address1 := 0x01CC4328
Address1CustomCode = 1
gosub %CurrentLoopCode%

return

BSProgressAddress1CustomCode:
Clothing := Memory(3, 0x01CC6B70+GameRunningAddress, 4) - 13	; as 13 is starting amount
SodaBuy := Memory(3, 0xC1A640, 2)								; Sodas Purchased
SodaDrink := Memory(3, 0xC1A3D8, 2)								; Sodas Drunk
onFoot := Floor(Memory(3, 0xC1A34C, 4, "Float")/10)/100			; Traveled on Foot
onBike := Floor(Memory(3, 0xC1A354, 4, "Float")/10)/100			; Traveled on Bike
PenaltyW := Memory(3, 0xC1A694, 2)								; Penalty Shootout Won
KeepUpsW := Memory(3, 0xC1A68C, 2)								; Keep-Ups won
FireAlarms := Memory(3, GameRunningAddress + 0x81A3B4, 2) 		; Fire Alarms pulled
ProjHits := Memory(3, GameRunningAddress + 0x81A4E8, 2)			; Projectile Hit


;Loha : 1 - Checklist , 2 - Chapter 1, 3 - Boxing, 4 - Chapter 2, 5 - Chapter 3 , 6 - Chapter 4, 7 - Chapter 5
MissionCounter := [0,0,0,0,0,0,0]
;Loha : 1 - Chemestry, 2 - English, 3 - Art, 4 - Gym, 5 - Shop, 6 - Photo, 7 - Music, 8 - Biology, 9 - Geo, 10 - Math, 11 - All, 12 - Failed
ClassCounter := [0,0,0,0,0,0,0,0,0,0,0,0]
;Loha : 1 - All, 2 - Transistors, 3 - Cards, 4 - Rubberbands, 5 - Tombstone, 6 - Pumpkins, 7 - Gnomes (To be added more)
CollectibleCounter := [0,0,0,0,0,0,0]

;Loha : 1 - Offset, 2 - If considered as checklist mission, 3 - Chapter/Calculation type (-1) (Mainly for Percentage calculation, might expand in future)
;Loha : Offsets themselfs are documented by me, but had no convientient way of copying which is which over
MissionOffsetArray := [[0x1A,0,1],[0x20,0,1],[0x26,1,1],[0x32,0,1],[0x38,1,1],[0x3E,1,1],[0x4A,1,1],[0x50,1,1],[0x56,1,1],[0x7A,1,1],[0x80,1,1],[0x8C,1,1],[0x9E,1,1],[0xBC,1,1],[0xC2,0,1],[0x5C,1,1],[0x62,1,1],[0x68,1,1],[0x6E,1,1],[0x74,1,1],[0x86,1,1],[0xB6,1,1]]
MissionOffsetArray.Push([0xCE,0,2],[0xD4,0,2],[0xDA,0,2],[0xE0,0,2],[0xE6,0,2],[0xEC,1,3],[0xF2,1,3],[0xF8,1,3],[0x104,1,3],[0x10A,1,3],[0x110,1,3],[0x116,1,3],[0x11C,1,3],[0x122,1,3],[0x128,1,3],[0x182,1,3],[0x188,1,3],[0x18E,1,3],[0x194,1,3],[0x19A,0,3])
MissionOffsetArray.Push([0x1B2,1,4],[0x1BE,1,4],[0x1C4,1,4],[0x1CA,1,4],[0x1D0,1,4],[0x1D6,1,4],[0x1DC,1,4],[0x1E2,1,4],[0x1E8,1,4],[0x1EE,1,4],[0x1F4,1,4],[0x1FA,1,4],[0x200,1,4],[0x206,1,4],[0x20C,1,4],[0x212,1,4],[0x218,0,4],[0x21E,1,4])
MissionOffsetArray.Push([0x22A,1,5],[0x230,1,5],[0x236,1,5],[0x23C,0,5],[0x242,1,5],[0x248,1,5],[0x24E,1,5],[0x254,1,5],[0x25A,1,5],[0x260,1,5],[0x266,1,5],[0x26C,1,5],[0x272,0,5])
MissionOffsetArray.Push([0x27E,1,6],[0x284,1,6],[0x28A,1,6],[0x290,1,6],[0x296,1,6],[0x29C,1,6],[0x2A2,1,6],[0x2A8,1,6],[0x2AE,1,6],[0x2B4,1,6],[0x2C6,1,6],[0x2CC,1,6],[0x2D2,1,6],[0x2D8,1,6],[0x2DE,1,6],[0x2E4,1,6],[0x2EA,0,6],[0x2F0,0,6])


MissionPointer := 0x01CC4328+GameRunningAddress
ClassOffsets := [0x302, 0x3A4, 0x326, 0x368, 0x3C8, 0x386, 0x476, 0x40A, 0x42E, 0x452]

;Loha : Some of the regions i probably could have handled same as i did with classes by doing Index * 6 , but ehh, more flexabilty this way i guess or smth ... ill be honest idk what im doing at times ...
;So i seperated the races here real quick
;1 - Offset, 2 - Location [1 - Vale, 2 - Town, 3 - Coventry, 4 - Academy, 5 - Circus(Kart), 6 - Town(Kart)]
RaceOffsetArray := [[0x12E,1],[0x134,1],[0x13A,1],[0x140,1],[0x146,1],[0x14C,1],[0x152,1],[0x158,2],[0x15E,2],[0x164,2],[0x16A,2],[0x170,3],[0x176,3],[0x17C,4],[0x4C4,5],[0x4CA,5],[0x4D0,5],[0x4D6,5],[0x4DC,5],[0x4E2,6],[0x4E8,6],[0x4EE,6]]
RaceLocation := [0,0,0,0,0,0]

;1 - Paper Route, 2 - Mowing (Park), 3 - Mowing(House)
JobOffsets := [[0xFE,1],[0x5D8,2],[0x5DE,2],[0x5E4,2],[0x5EA,3],[0x5F0,3],[0x5F6,3]]
JobCounter := [0,0,0]

For i in JobOffsets{
		If(Memory(3, Memory(5, MissionPointer, JobOffsets[i][1]),1) > 0){
		JobCounter[JobOffsets[i][2]]++
	}
}

For i in RaceOffsetArray{
	If(Memory(3, Memory(5, MissionPointer, RaceOffsetArray[i][1]),1) > 0){
		RaceLocation[RaceOffsetArray[i][2]]++
	}
}

For i in MissionOffsetArray{
	If(Memory(3, Memory(5, MissionPointer, MissionOffsetArray[i][1]),1) > 0){
		CounterIndex := MissionOffsetArray[i][3] + 1
		MissionCounter[CounterIndex]++
	}
	If(Memory(3, Memory(5, MissionPointer, MissionOffsetArray[i][1]),1) > 0 && MissionOffsetArray[i][2] > 0){
		MissionCounter[1]++
	}
}

For i in ClassOffsets{
	Loop, 6{
		ClassCountID := A_Index - 1
		if (ClassCountID == 5 && (i == 4 || i == 6)) {				;Gym and Photo classes don't have a 6th class variant 
			break
		}
		ClassCompletion := Memory(3,Memory(5, MissionPointer, ClassOffsets[i]+(0x6*ClassCountID)),1)
		ClassStatus := Memory(3,Memory(5, MissionPointer, ClassOffsets[i]+0x2+(0x6*ClassCountID)),1)
		ClassAttempt := Memory(3,Memory(5, MissionPointer, ClassOffsets[i]-0x2+(0x6*ClassCountID)),1)
		If( ClassCompletion > 0){
			ClassCounter[i]++
			ClassCounter[11]++
		}
		ClassCounter[12] := ClassCounter[12] + ClassAttempt - ClassCompletion ;Calculates failed classes by adding attempts and subtracting succesful attempts
		If( ClassStatus == 17 ){
			ClassCounter[12]--						;Removes ongoing class attempt 
		}
	}
}

;Loha : Each Collectible loop thing... im bad at commenting stuff okay...
;Loha : Also Zero'd the index as it just makes it easier for me to copy stuff over from C++
Loop, 192{
	Counter := A_Index - 1
	CollectibleAddress := GameRunningAddress + 0x007CEB54 + (2*Counter)
	If(Memory(3, CollectibleAddress, 1) > 0){
		CollectibleCounter[1]++
		if(Counter >= 0 && Counter <= 24){	;Gnomes
			CollectibleCounter[7]++
		}
		if(Counter >= 25 && Counter <= 30){	;Transistors
			CollectibleCounter[2]++
		}
		if(Counter >= 31 && Counter <= 105){	;Rubberbands
			CollectibleCounter[4]++
		}
		if(Counter >= 106 && Counter <= 145){	;Cards
			CollectibleCounter[3]++
		}
		if(Counter >= 146 && Counter <= 172){	;Pumpkins
			CollectibleCounter[6]++
		}
		If(Counter >= 173 && Counter <= 191){	;Tombstones
			CollectibleCounter[5]++
		}
	}
}

;Errands
ErrandCounter := 0
Loop, 50{
	Counter := A_Index - 1
	ErrandAddress := GameRunningAddress + 0x81B096 + (4*Counter)
	If(Memory(3, ErrandAddress, 1) > 0){
		ErrandCounter++
	}
}

; 1 - All, 2 - Boxing, 3 - Carnival Games, 4 - Carnival Rides
ActivityCounter := [MissionCounter[3],MissionCounter[3],0,0]

; Keep-ups / Penalty Shootout being added to Boxing counter, as uses same % calculation
If(KeepUpsW > 0){
	ActivityCounter[1]++
}
If(PenaltyW > 0){
	ActivityCounter[1]++
}
; Carnival Games
; Strike Out, High Striker, Splish Splash and Shooting Range.
CarnivalGame := [0x4B2,0x4AC,0x4BE,0x4B8]
For i in CarnivalGame{
	if(Memory(3, Memory(5, MissionPointer, CarnivalGame[i]),1) > 0){
		ActivityCounter[1]++
		ActivityCounter[3]++
	}
}
; Carnival Rides
;The Ferris Wheel, The Big Squid and Big Canyon Railway
CarnivalRide := [0x4A6,0x4A0,0x49A]
For i in CarnivalRide{
	if(Memory(3, Memory(5, MissionPointer, CarnivalRide[i]),1) > 0){
		ActivityCounter[1]++
		ActivityCounter[4]++
	}
}

; Arcade Games
ArcadeCounter = 0
BConSumo := Memory(3,GameRunningAddress+0x81A6A0,4)
If(BConSumo > 1010){
	ArcadeCounter++
}
BNut := Memory(3,GameRunningAddress+0x81A6B8,4)
If(BNut > 69900){
	ArcadeCounter++
}
BMonkey := Memory(3,GameRunningAddress+0x81A6AC,4)
If(BMonkey > 333){
	ArcadeCounter++
}

; Locker clothing piece check
; Bandit Mask - (70 BE 49 49) [00 (58 36 3C)]
; Werewolf Mask - (FD 95 6F 76) [00 (58 36 3C)]
; 				(Determines what is what)[Equip Slot (Method of obtaining... doesn't carry over in saves, nor doesn't get reset)]
BullyPirate := 0
LockerCounter := 0
AllPurchasableClothes := 0
Loop, 362{
	IndexCounter := A_Index - 1
	WardrobeSlot := 0x20C6020 + (8*IndexCounter)
	ClothingValue := Memory(3, WardrobeSlot, 4)
	If(ClothingValue == 1987024381 or ClothingValue == 1229569648){
		LockerCounter++
	}
	else if(ClothingValue == 623211024){
		BullyPirate++
	}
	else if(ClothingValue == 356060560){
		AllPurchasableClothes++
	}
}

SetFormat, Float, 0.20

; Yearbook - 500IQ Coding here ...
; 			( I legit feel that there is a WAY better method of achieving end result but ... this works)
YearbookCounter := 0
;No really ... i feel like this could have been done better
HexBinValue := {0:0,1:1,2:1,3:2,4:1,5:2,6:2,7:3,8:1,9:2,10:2,11:3,12:2,13:3,14:3,15:4}
Loop, 8{
	IndexCounter := A_Index - 1
	ByteValue := Memory(3, 0xC1B008 + (0x1*IndexCounter), 1)
	Bit1 := ByteValue/16
	Bit2 := (Bit1 - Floor(Bit1))*16
	YearbookCounter := YearbookCounter + HexBinValue[Floor(Bit1)] + HexBinValue[Floor(bit2)]
}

SetFormat, Float, 0.%DecimalPlaces%

ArcadeP := ArcadeCounter * 1
MissionP := (Floor(((MissionCounter[2]) * 0.27275)*100)/100) + (Floor(((ActivityCounter[1]) * 0.3572)*100)/100) + (Floor(((MissionCounter[4]) * 0.5335)*100)/100) + (Floor(((MissionCounter[5]) * 0.5556)*100)/100) + (Floor(((MissionCounter[6]) * 0.9233)*100)/100) + (Floor(((MissionCounter[7]) * 0.7778)*100)/100)
ClassP := ClassCounter[11]*0.20
CollectibleP := Floor((CollectibleCounter[1] * 0.04167)*100)/100
ClothingP := Floor((Clothing * 0.0148815)*100)/100
RaceP := (Floor(((RaceLocation[1]+RaceLocation[2]+RaceLocation[3]+RaceLocation[4])* 0.4286)*100)/100) + (Floor(((RaceLocation[5]+RaceLocation[6])* 0.5)*100)/100)
JobP := (Floor(((JobCounter[1]+JobCounter[2]+JobCounter[3])* 0.2858)*100)/100)
ErrandP := ErrandCounter * 0.04

;Misc % ( Mostly secrets as it seems)
MiscCounter := 0

If(Memory(3, Memory(5, MissionPointer, 0xB0),1) > 0){			;This Address doesn't determine the 0.23% but it changes same time
	MiscCounter++
}
If(CollectibleCounter[6] == 27){								;All Pumpkins obtained.
	MiscCounter++
}
If(CollectibleCounter[5] == 19){								;Edna mask ( Tombstones )
	MiscCounter++
}
If(onFoot >= 100){												;100Km walked
	onFoot = 100 	;To prevent decimals showing after 100 and breaking the output UI
	MiscCounter++
}
If(onBike >= 100){												;100Km on bike
	onBike = 100 	;To prevent decimals showing after 100 and breaking the output UI
	MiscCounter++
}
If(SodaBuy >= 100){												;100 Purchased Sodas
	MiscCounter++
}
If(SodaDrink >= 500){											;500 Drank Sodas
	MiscCounter++
}
If(FireAlarms >= 20){											;20 Fire Alarms Pulled
	MiscCounter++
}
If(ProjHits >= 1000){											;1000 ProjectileHits
	MiscCounter++
}
If(ArcadeCounter >= 3){											;All Arcade HiScores
	MiscCounter++
}
If(RaceLocation[5] >= 1){											;First GoKart race
	MiscCounter++
}
If(ClassCounter[12] >= 5){										;Dunce cap
	MiscCounter++
}
If(BullyPirate >= 1){											;Secret Pirate
	MiscCounter++
}
If(YearbookCounter >= 61){										;Yearbook completed
	MiscCounter++
}
If(CollectibleCounter[3] >= 40){										;Cards Collected
	MiscCounter++
}
If(CollectibleCounter[4] >= 75){										;Rubberbands Collected
	MiscCounter++
}
If(CollectibleCounter[7] >= 25){										;Gnomes Smashed
	MiscCounter++
}
If(ClassCounter[11] >= 50){										;All Classes completed
	MiscCounter++
}
If(AllPurchasableClothes >= 1){										;All Purchased clothes obtained
	MiscCounter++
}
If(CollectibleCounter[1] >= 192){										;All Collectebles
	MiscCounter++
}

MiscP := Floor((MiscCounter * 0.238)*100)/100

MemoryValue := MissionP + ClothingP + ClassP + CollectibleP + MiscP + RaceP + JobP + ErrandP + ArcadeP
return

BullyMissionsAddress1CustomCode:
MemoryValue := MissionCounter[1]
return

BullyErrandsAddress1CustomCode:
MemoryValue := ErrandCounter
return

BullyChemAddress1CustomCode:
MemoryValue := ClassCounter[1]
return

BullyEnglishAddress1CustomCode:
MemoryValue := ClassCounter[2]
return

BullyArtAddress1CustomCode:
MemoryValue := ClassCounter[3]
return

BullyGymAddress1CustomCode:
MemoryValue := ClassCounter[4]
return

BullyBiologyAddress1CustomCode:
MemoryValue := ClassCounter[8]
return

BullyMusicAddress1CustomCode:
MemoryValue := ClassCounter[7]
return

BullyShopAddress1CustomCode:
MemoryValue := ClassCounter[5]
return

BullyPhotoClassAddress1CustomCode:
MemoryValue := ClassCounter[6]
return

BullyGeoAddress1CustomCode:
MemoryValue := ClassCounter[9]
return

BullyMathAddress1CustomCode:
MemoryValue := ClassCounter[10]
return

BullyDunceAddress1CustomCode:
MemoryValue := ClassCounter[12]
return

BullyYearbookAddress1CustomCode:
MemoryValue := YearbookCounter
return

BullyBoxingAddress1CustomCode:
;Loha : This wasn't part of boxing ... uses same % calculation tho /shrug
If(Memory(3, Memory(5, MissionPointer,0xE6),1) > 0){
	ActivityCounter[2]--
}
MemoryValue := ActivityCounter[2]
return

BullyKeepUpAddress1CustomCode:
MemoryValue := KeepUpsW
return

BullyPenShootAddress1CustomCode:
MemoryValue := PenaltyW
return

BullyPRouteAddress1CustomCode:
MemoryValue := JobCounter[1] + Memory(3,Memory(5, MissionPointer, 0xF8), 2)
return

BullyLMParkAddress1CustomCode:
MemoryValue := JobCounter[2]
return

BullyLMHouseAddress1CustomCode:
MemoryValue := JobCounter[3]
return

BullyValeBRAddress1CustomCode:
MemoryValue := RaceLocation[1]
return

BullyTownBRAddress1CustomCode:
MemoryValue := RaceLocation[2]
return

BullyCoveBRAddress1CustomCode:
MemoryValue := RaceLocation[3]
return

BullyAcaBRAddress1CustomCode:
MemoryValue := RaceLocation[4]
return

BullyGPRAddress1CustomCode:
MemoryValue := RaceLocation[5]
return

BullyGOStreetAddress1CustomCode:
MemoryValue := RaceLocation[6]
return

BullyCarGamesAddress1CustomCode:
MemoryValue := ActivityCounter[3]
return

BullyCarnRidesAddress1CustomCode:
MemoryValue := ActivityCounter[4]
return

BullyMonkeyHSAddress1CustomCode:
MemoryValue := BMonkey
return

BullySumoHSAddress1CustomCode:
MemoryValue := BConSumo
return

BullyNutHSAddress1CustomCode:
MemoryValue := BNut
return

BullyCloAddress1CustomCode:
MemoryValue := Clothing
return

BullyLockerCloAddress1CustomCode:
MemoryValue := LockerCounter
return

BullyPirateAddress1CustomCode:
MemoryValue := BullyPirate
return

BullyFireAlarmAddress1CustomCode:
MemoryValue := FireAlarms
return

BullyGnomeAddress1CustomCode:
MemoryValue := CollectibleCounter[7]
return

BullyRadioAddress1CustomCode:
MemoryValue := CollectibleCounter[2]
return

BullyRubberAddress1CustomCode:
MemoryValue := CollectibleCounter[4]
return

BullyCardsAddress1CustomCode:
MemoryValue := CollectibleCounter[3]
return

BullyTombAddress1CustomCode:
MemoryValue := CollectibleCounter[5]
return

BullyPumpAddress1CustomCode:
MemoryValue := CollectibleCounter[6]
return

BullySodaPurchaseAddress1CustomCode:
MemoryValue := SodaBuy
return

BullySodaAddress1CustomCode:
MemoryValue := SodaDrink
return

BullyOnFootAddress1CustomCode:
MemoryValue := onFoot
return

BullyOnBikeAddress1CustomCode:
MemoryValue := onBike
return

BullyProjHitsAddress1CustomCode:
MemoryValue := ProjHits
return

; ######################################################################################################

DebugFunctions:
Hotkey, F7, DebugListvars, On
return

DebugListvars:
listvars
return
